#!/usr/bin/env jython
import string
import re

class Method:
    def __init__ (self, name, parameters, returnType, doc):
        self.name = name
        self.parameters = parameters
        self.returnType = returnType
        self.doc = doc

class Constant:
    def __init__ (self, name, value, doc):
        self.name = name
        self.value = value
        self.doc = doc

class InterfaceParser:
    def __init__ (self, doc = None, methods = None, constants = None):
        self.doc = None
        if methods is None:
            methods = []
        self.methods = methods
        if constants is None:
            constants = []
        self.constants = constants

    identifier = r'(\w+)'
    interfaceRE = re.compile (r'\s*public\s+interface\s+' + identifier)
    constRE = re.compile (r'\s*int\s+' + identifier + r'\s*=\s*(\d+)')
    methodRE = re.compile (r'\s*' + identifier + r'\s+' + identifier
        + r'\s*\(([^)]*)\)')
    def parseFile (self, fname):
        source = open (fname, 'rt').read ()
        pos = 0
        eoSource = len (source)
        while (pos) < eoSource:
            pos, comment = self.nextComment (source, pos)
            if not comment:
                break
            for rex, action in [(self.interfaceRE, self.addInterface),
                    (self.constRE, self.addConstant),
                    (self.methodRE, self.addMethod)]:
                match = rex.match (source, pos)
                if match:
                    action (match, self.undecorateComment (comment))
                    break
        self.constants = sortByName (self.constants)
        self.methods = sortByName (self.methods)

    def nextComment (self, source, pos):
        commentPos = source.find ('/*', pos)
        if commentPos == -1:
            return None, None
        commentPos += 2
        eoComment = source.index ('*/', commentPos)
        comment = source [commentPos:eoComment]
        lines = comment.split ('\n')
        newPos = eoComment + 2
        assert newPos > pos
        return newPos, lines

    decorRE = re.compile (r'\s*[*]\s*(.*)')
    def undecorateComment (self, comment):
        if not comment:
            return comment
        result = []
        rex = self.decorRE
        for line in comment:
            match = rex.match (line)
            if match:
                line = match.group (1)
            result.append (line)
        if not result [-1].strip ():
            del result [-1]
        return result

    def addInterface (self, match, comment):
        self.doc = comment

    def addConstant (self, match, comment):
        name, value = match.group (1, 2)
        constant = Constant (name, value, comment)
        self.constants.append (constant)

    def addMethod (self, match, comment):
        name, returnType, parameterString = match.group (2, 1, 3)
        parameters = []
        for oneString in parameterString.split (','):
            list = map (string.strip, oneString.split ())
            if not list:
                continue
            type, pname = list
            parameters.append ((type, pname))
        method = Method (name.strip (), parameters, returnType.strip (), comment)
        self.methods.append (method)

    def writeCache (self, fname):
        stream = open (fname, 'wt')
        stream.write ('\nfrom InterfaceParser import InterfaceParser, Method, Constant\n\n')
        stream.write ('DatabaseMetadataInterface = InterfaceParser (\n')
        stream.write ('    doc = ' + `self.doc`)
        if self.methods:
            stream.write (',\n    methods = [\n')
            for method in self.methods:
                stream.write ('        Method (' + `method.name`
                    + ', ' + `method.parameters`
                    + ', ' + `method.returnType`
                    + ', ' + `method.doc` + '),\n')
            stream.write ('   ]')
        if self.constants:
            stream.write (',\n    constants = [\n')
            for constant in self.constants:
                stream.write ('        Constant (' + `constant.name`
                    + ', ' + `constant.value`
                    + ', ' + `constant.doc` + '),\n')
            stream.write ('   ]')
        stream.write (')\n')
        stream.close ()


def sortByName (list):
    sortList = [(e.name, e) for e in list]
    sortList.sort ()
    return [e for name, e in sortList]

def main (options, args):
    parser = InterfaceParser ()
    parser.parseFile (options.inputfile)
    parser.writeCache (options.cache)

def _options ():
    return [
        # (optstring, varname, typechar, default, help)
        ('i', 'inputfile', ':', 'DatabaseMetadata.java', 'name of input file'),
        ('cache', None, ':', 'Interface.cache', 'file to cache interface definition'),
        ]

if __name__ == '__main__':
    import optlib
    optlib.optMain2 (main, _options ())

