#!/usr/bin/env python
# genMetadataClass.py

import sys
import os

try:
    import java.lang
    import java.sql
    hasJava = 1
except:
    hasJava = None

class ClassGenerator:
    basicTypes = ['int', 'boolean', 'String']
    ignoreMethods = ['getURL', 'getUserName', 'getDriverMajorVersion',
        'getDriverMinorVersion', 'getDriverName', 'getDriverVersion']
    def gen (self, options, interfaceInfo, jdbcmeta, overrides):
        stream = open (options.outfile, 'wt')
        stream.write ('\n')
        #
        # intro
        #
        stream.write ('true = \'true\'\n')
        stream.write ('false = None\n')
        stream.write ('\n')
        stream.write ('class DatabaseMetadata:\n')
        self.writeDoc (stream, interfaceInfo.doc, '    ')
        init = overrides.get ('__init__', '')
        stream.write (init)
        #
        # constants
        #
        for constant in interfaceInfo.constants:
            if constant.doc:
                self.writeComment (stream, constant.doc, '    ')
            stream.write ('    ' + constant.name + ' = ' + constant.value + '\n')
            if constant.doc:
                stream.write ('\n')
        #
        # methods
        #
        for method in interfaceInfo.methods:
            stream.write ('    def ' + method.name + ' (self')
            for type, name in method.parameters:
                stream.write (', ' + name)
            stream.write ('):\n')
            self.writeDoc (stream, method.doc, '        ')
            code = self.getMethodCode (method, jdbcmeta, overrides, '        ')
            if code:
                stream.write (code)
            else:
                stream.write ('        pass\n')
            stream.write ('    # end ' + method.name + '\n')
            if not code:
                stream.write ('    del ' + method.name + '\n')
                print 'no code for:', method.name
            stream.write ('\n')
        #
        #
        #
        stream.close ()

    def writeDoc (self, stream, doc, indent):
        if not doc:
            return
        stream.write (indent + '"""' + doc [0] + '\n')
        for line in doc [1:]:
            stream.write (indent + line + '\n')
        stream.write (indent + '"""\n')

    def writeComment (self, stream, doc, indent):
        if not doc:
            return
        stream.write (indent + '# ' + doc [0] + '\n')
        for line in doc [1:]:
            stream.write (indent + '# ' + line + '\n')

    def getMethodCode (self, method, jdbcmeta, overrides, indent):
        # look in overrides
        if overrides.has_key (method.name):
            return overrides [method.name]
        if len (method.parameters) > 0:
            return None
        if not jdbcmeta:
            return None
        if method.returnType not in self.basicTypes:
            return None
        if method.name in self.ignoreMethods:
            return None
        try:
            javaMethod = getattr (jdbcmeta, method.name)
            result = javaMethod ()
            if method.returnType == 'boolean':
                if result:
                    result = 'true'
                else:
                    result = 'false'
            else:
                result = `result`
            return indent + 'return ' + result + '\n'
        except:
            excClass, exc, traceback = sys.exc_info ()
            print exc
        return None

def loadFromFile (filename, variable):
    globals = {}
    execfile (filename, globals)
    return globals [variable]

def loadInterfaceInfo (options):
    return loadFromFile (options.cache, 'DatabaseMetadataInterface')

def loadOverrides (options):
    if os.path.exists (options.overrides):
        return loadFromFile (options.overrides, 'overrides')
    else:
        return {}

def loadJdbcmeta (options):
    if not hasJava:
        return None
    for option in ['jdbcclass', 'url', 'username', 'password']:
        if not getattr (options, option):
            print 'skipping JDBC connect because option -' + option + ' is missing'
            return None
    java.lang.Class.forName (options.jdbcclass)
    session = java.sql.DriverManager.getConnection (
        options.url, options.username, options.password)
    jdbcmeta = session.getMetaData ()
    return jdbcmeta

def genMetadataClass (options):
    interface = loadInterfaceInfo (options)
    overrides = loadOverrides (options)
    jdbcmeta = loadJdbcmeta (options)
    ClassGenerator ().gen (options, interface, jdbcmeta, overrides)

def main (options, args):
    genMetadataClass (options)
    print 'wrote', options.outfile

def _options ():
    return [
        # (optstring, varname, typechar, default, help)
        ('o', 'outfile', ':', 'DatabaseMetadata.py', 'name of output file'),
        ('cache', None, ':', 'Interface.cache', 'file to cache interface definition'),
        ('overrides', None, ':', 'DatabaseMetada.overrides', 'file with overriding definitions'),
        ('jdbcclass', None, ':', None, 'name of class to load JDBC driver'),
        ('url', None, ':', None, 'url to connect to database'),
        ('username', None, ':', None, 'username to connect to database'),
        ('password', None, ':', None, 'password to connect to database'),
        ]

if __name__ == '__main__':
    import optlib
    optlib.optMain2 (main, _options ())
